package mastermind.client;

import java.util.Scanner;
import mastermind.engine.MastermindEngine;
import mastermind.logic.ArgParser;
import mastermind.logic.GuessHandler;
import mastermind.logic.RandomGuess;
import mastermind.logic.SettingsHandler;
import mastermind.model.Guess;
import mastermind.model.Settings;

/**
 * Mastermind Client Console.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-04.v01   FeRo    init version
 * 2012-05-05.v01   FeRo    cmd args, settings, div Code optimierungen
 * 2012-05-10.v01   FeRo    -? Usage erweitert
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-10.v01
 */
public class ClientConsole implements IClient
{

    private SettingsHandler settingsHandler;
    private Scanner console;

    /**
     * Default Konstruktor - empty.
     */
    public ClientConsole()
    {
        // "standart" input stream
        console = new Scanner(System.in);
        settingsHandler = SettingsHandler.getInstance();
    }

    /**
     * Starten den Client
     * @param mEngine Mastermind Engine
     * @param args Cmd Args
     */
    @Override
    public void runClient(MastermindEngine mEngine, String[] args)
    {
        // Show Welcome Banner
        showBanner();

        // cmd args
        ArgParser parser = new ArgParser(args);
        if (parser.hasOption("help") || parser.hasOption("h") || parser.hasOption("?")) {
            // Show Help, Usage
            showUsage();
        } else {
            // Get Settings from User / Console
            setSettings(parser);

            // Debug Hidden Code
            boolean showSecretCode = false;
            if (parser.hasOption("*")) {
                showSecretCode = true;
            }

            // Set hidden Pattern
            String secretPattern;
            boolean generateRandomPattern = true;
            if (parser.hasOption("s")) {
                secretPattern = parser.getOption("s");
                if (GuessHandler.validateGuess(MastermindEngine.convertStringToIntArr(secretPattern))) {
                    generateRandomPattern = false;
                    // Use cmd Pattern
                    mEngine.setHiddenPattern(new Guess(MastermindEngine.convertStringToIntArr(secretPattern)), showSecretCode);
                }
            }
            if (generateRandomPattern) {
                // Generate Random Pattern
                System.out.println("Secret Code wird erzeugt...");
                mEngine.setHiddenPattern(RandomGuess.genRandomGuess(), showSecretCode);
            }

            // Play
            boolean askPlayAgain = true;
            if (parser.hasOption("n")) {
                askPlayAgain = false;
            }
            doPlay(mEngine, askPlayAgain, showSecretCode);
        }
    }

    private void doPlay(MastermindEngine mEngine, boolean askPlayAgain, boolean showSecretCode)
    {
        // let's play :D
        Guess myGuess;
        String guess;
        int countRound;
        int countPlayRound = 0;
        boolean doPlay = true;
        boolean doPlayAgain = false;
        boolean vallidSettings;

        do { // playAgain
            countPlayRound ++;
            System.out.println("-------------------------------------------------------------------------------");
            if (countPlayRound > 1) {
                // Generate new Hidden Pattern!
                System.out.println("Secret Code wird erzeugt...");
                mEngine.setHiddenPattern(RandomGuess.genRandomGuess(), showSecretCode);
                // System.out.println("\nViel Glueck!");
            }

            countRound = 0;
            doPlay = true;
            doPlayAgain = false;
            do { // doPlay
                countRound ++;
                do {
                    vallidSettings = true;
                    System.out.print("Versuch " + countRound + "/" + settingsHandler.getSettings().getNumberOfRows() + ": ");
                    guess = getConsoleString("");
                    myGuess = new Guess(MastermindEngine.convertStringToIntArr(guess));
                    vallidSettings = GuessHandler.validateGuess(myGuess);
                    if ( ! vallidSettings) {
                        System.out.println(" > Bitte eine gueltige Kombination eingeben!");
                    }
                } while ( ! vallidSettings);

                // have we won?
                if (mEngine.hasWon(myGuess) || (countRound >= settingsHandler.getSettings().getNumberOfRows())) {
                    doPlay = false;
                } else {
                    // get Result: Black / White pins
                    System.out.println(" > " + mEngine.getResult(myGuess));
                }
            } while (doPlay);


            if (mEngine.hasWon(myGuess)) {
                System.out.println("\nGratulation! Du hast gewonnen!");
            } else {
                System.out.println("\nDu hast leider nicht gewonnen. Die gesuchte Kombination war: " + mEngine.getHiddenPattern());
            }

            if (askPlayAgain) {
                String strPlayAgain;
                final String defaultPlayAgain = "j";
                do {
                    vallidSettings = true;
                    System.out.print("\nWillst Du nochmals spielen? (j/n) [" + defaultPlayAgain + "]: ");
                    strPlayAgain = getConsoleString(defaultPlayAgain).toLowerCase();
                    if (strPlayAgain.equals("j")) {
                        doPlayAgain = true;
                    } else if (strPlayAgain.equals("n")) {
                        doPlayAgain = false;
                    } else {
                        vallidSettings = false;
                        System.out.println(" > Bitte [j] oder [n] eingeben!");
                    }
                } while ( ! vallidSettings);
            } // askPlayAgain

        } while (doPlayAgain);
    }

    private int getArgsInteger(ArgParser parser, String option, int errorValue)
    {
        int res = errorValue;
        if (parser.hasOption(option)) {
            try {
                res = Integer.parseInt(parser.getOption(option));
            } catch (Exception e) {
                System.out.println("Error: " + e.getLocalizedMessage());
                res =
                    errorValue;
            }
        }
        return res;
    }

    /**
     * Hilfsmethode um einen Integer von der Console zu lesen
     * @param defaultValue Default Wert falls [Enter]
     * @return eingelesener Integer Wert
     */
    private int getConsoleInteger(int defaultValue)
    {
        String strInput = "";
        int intRes = defaultValue;
        try {
            strInput = console.nextLine();
        } catch (java.util.NoSuchElementException e) {
            // Ctrl-C
            System.exit(0);
        } catch (Exception e) {
            // ignore...
            //System.out.println("Error: " + e.getLocalizedMessage());
        }

        if ( ! strInput.equals("")) {
            intRes = -1;
            try {
                intRes = Integer.parseInt(strInput);
            } catch (Exception e) {
                // ignore...
                //System.out.println("Error: " + e.getLocalizedMessage());
            }
        }
        return intRes;
    }

    /**
     * Hilfsmethode um einen Integer von der Console zu lesen
     * @param defaultValue Default Wert falls [Enter]
     * @return eingelesener String Wert
     */
    private String getConsoleString(String defaultValue)
    {

        String strInput = "";
        try {
            strInput = console.nextLine();
        } catch (java.util.NoSuchElementException e) {
            // Ctrl-C
            System.exit(0);
        } catch (Exception e) {
            // ignore...
            //System.out.println("Error: " + e.getLocalizedMessage());
        }

        if (strInput.equals("") &&  ! defaultValue.equals("")) {
            strInput = defaultValue;
        }
        return strInput;
    }

    /**
     * List die Optionen von der Konsole ein und setzt die Settings, Spiel Optionen
     * @param parser ArgumentParser für die cmd parameter
     */
    public void setSettings(ArgParser parser)
    {
        Settings settings = SettingsHandler.getInstance().getSettings();

        final int defaultNumberOfColor = 4;
        final String defaultAllowDuplicateColor = "n";
        final int defaultNumberOfPegs = 4;
        final int defaultNumberOfRows = 6;
        boolean vallidSettings = true;
        int numberOfColor = defaultNumberOfColor;
        String allowDuplicateColor = defaultAllowDuplicateColor;
        int numberOfPegs = defaultNumberOfPegs;
        int numberOfRows = defaultNumberOfRows;

        do {
            // Anzahl Farben CMD Args
            vallidSettings = true;
            if (parser.hasOption("c")) {
                numberOfColor = getArgsInteger(parser, "c", -1);
                try {
                    settings.setNumberOfColors(numberOfColor);
                } catch (Exception e) {
                    vallidSettings = false;
                }
                if (vallidSettings) {
                    System.out.println("Anzahl Farben:                 " + numberOfColor);
                }
            } else {
                vallidSettings = false;
            }
            // Anzahl Farben User Input
            while ( ! vallidSettings) {
                vallidSettings = true;
                System.out.print("Anzahl Farben (" + Settings.ANZ_COLOR_MIN + "-" + Settings.ANZ_COLOR_MAX + ") [" + defaultNumberOfColor + "]:       ");
                numberOfColor = getConsoleInteger(defaultNumberOfColor);
                try {
                    settings.setNumberOfColors(numberOfColor);
                } catch (Exception e) {
                    vallidSettings = false;
                    System.out.println(" > Bitte gueltige anzahl Farben eingeben!");
                }

            }

            // Doppelte Farben Ja/Nein
            vallidSettings = true;
            if (parser.hasOption("d")) {
                settings.setAllowDuplicateColor(true);
                System.out.println("Doppelte Farben:               Ja");
            } else {
                if (parser.numberOfParams() > 0) {
                    settings.setAllowDuplicateColor(false);
                    System.out.println("Doppelte Farben:               Nein");
                } else {
                    vallidSettings = false;
                }
            }

            // Doppelte Farben User Input
            while ( ! vallidSettings) {
                vallidSettings = true;
                System.out.print("Doppelte Farben (j/n) [" + defaultAllowDuplicateColor + "]:     ");
                allowDuplicateColor = getConsoleString(defaultAllowDuplicateColor).toLowerCase();
                if (allowDuplicateColor.equals("j")) {
                    settings.setAllowDuplicateColor(true);
                } else if (allowDuplicateColor.equals("n")) {
                    settings.setAllowDuplicateColor(false);
                } else {
                    vallidSettings = false;
                    System.out.println(" > Bitte [j] oder [n] eingeben!");
                }
            }
            // Anzahl Steckplätze
            vallidSettings = true;
            if (parser.hasOption("p")) {
                numberOfPegs = getArgsInteger(parser, "p", -1);
                //System.out.println("Args: numberOfPegs: " + numberOfPegs);
                try {
                    settings.setNumberOfPegs(numberOfPegs);
                } catch (Exception e) {
                    vallidSettings = false;
                }
                if (vallidSettings) {
                    System.out.println("Anzahl Steckplaetze:           " + numberOfPegs);
                }
            } else {
                vallidSettings = false;
            }
            // Anzahl Steckplätze User Input
            while ( ! vallidSettings) {
                vallidSettings = true;
                System.out.print("Anzahl Steckplaetze (" + Settings.ANZ_PEG_MIN + "-" + Settings.ANZ_PEG_MAX + ") [" + defaultNumberOfPegs + "]: ");
                numberOfPegs = getConsoleInteger(defaultNumberOfPegs);
                try {
                    settings.setNumberOfPegs(numberOfPegs);
                } catch (Exception e) {
                    vallidSettings = false;
                    System.out.println(" > Bitte gueltige anzahl Steckplaetze eingeben!");
                }
            }
            // Anzahl Versuche
            vallidSettings = true;
            if (parser.hasOption("a")) {
                numberOfRows = getArgsInteger(parser, "a", -1);
                //System.out.println("Args: numberOfRows: " + numberOfRows);
                try {
                    settings.setNumberOfRows(numberOfRows);
                } catch (Exception e) {
                    vallidSettings = false;
                }
                if (vallidSettings) {
                    System.out.println("Anzahl Versuche:               " + numberOfRows);
                }
            } else {
                vallidSettings = false;
            }
            // Anzahl Versuche User Input
            while ( ! vallidSettings) {
                vallidSettings = true;
                System.out.print("Anzahl Versuche (" + Settings.ANZ_GUESS_MIN + "-" + Settings.ANZ_GUESS_MAX + ") [" + defaultNumberOfRows + "]:    ");
                numberOfRows = getConsoleInteger(defaultNumberOfRows);
                try {
                    settings.setNumberOfRows(numberOfRows);
                } catch (Exception e) {
                    vallidSettings = false;
                    System.out.println(" > Bitte gueltige anzahl Versuche eingeben!");
                }
            }
            // Ist diese Kombination gültig / Spielbar?
            vallidSettings = true;
            if ( ! settingsHandler.validSettings()) {
                vallidSettings = false;
                System.out.println("\nDiese Optionen sind nicht gueltig! Bitte neue eingeben...");
            }
        } while ( ! vallidSettings);
    }

    private void showBanner()
    {
        System.out.println("\n\n");
        System.out.println("===============================================================================");
        System.out.println("HSLU Technik & Architektur                              Programmieren 2 | F1203");
        System.out.println("           __  __           _                      _           _ ");
        System.out.println("          |  \\/  | __ _ ___| |_ ___ _ __ _ __ ___ (_)_ __   __| |");
        System.out.println("          | |\\/| |/ _` / __| __/ _ \\ '__| '_ ` _ \\| | '_ \\ / _` |");
        System.out.println("          | |  | | (_| \\__ \\ |_  __/ |  | | | | | | | | | | (_| |");
        System.out.println("          |_|  |_|\\__,_|___/\\__\\___|_|  |_| |_| |_|_|_| |_|\\__,_|");
        System.out.println("");
        System.out.println("                  Redzepi Iljasa | Reichmuth Marco | Rey Philipp | Rohrer Felix");
        System.out.println("===============================================================================");
    }

    private void showUsage()
    {
        System.out.println("\nUsage: mastermind [-c=(count)]  [-d=(j/n)]  [-p=(count)]  [-a=(count)]");
        System.out.println("                  [-s=(secretcode)]  [-n=(j/n)]  [-?]");
        System.out.println("\nOptions:");
        System.out.println("   -c   Number of colors [" + Settings.ANZ_COLOR_MIN + "-" + Settings.ANZ_COLOR_MAX + "]");
        System.out.println("   -d   Allow duplicate color [j/n]");
        System.out.println("   -p   Number of pegs [" + Settings.ANZ_PEG_MIN + "-" + Settings.ANZ_PEG_MAX + "]");
        System.out.println("   -a   Number of attempts [" + Settings.ANZ_GUESS_MIN + "-" + Settings.ANZ_GUESS_MAX + "]");
        System.out.println("   -s   Use the specified secretcode [{secretcode}]");
        System.out.println("   -n   Do not ask for play again [j/n]");
        System.out.println("   -?   Show this help");
    }
}
